unit DeloneTriangulation;

interface

type
TTPoint=record
   x,y:integer;
   end;
TRib=record
   p1,p2:integer;
   end;
TTriangle=record
   p1,p2,p3:integer;
   end;
TCircle=record
   xo,yo,R:single;
   end;

var
  Points:array [0..16383] of TTPoint;
  PointsCount:integer;
  Ribs:array [0..16383] of TRib;
  RibsCount:integer;
  Triangles:array [0..16383] of TTriangle;
  TrianglesCount:integer;

procedure Triangulation;  

implementation

function Side(i,j,k:integer):integer;
var x1,y1,x2,y2,xo,yo,dx,dy,a,b,v:single;
begin
x1:=Points[i].x;
y1:=Points[i].y;
x2:=Points[j].x;
y2:=Points[j].y;
xo:=Points[k].x;
yo:=Points[k].y;
dx:=x2-x1;
dy:=y2-y1;
if abs(dx)>abs(dy) then
	 begin
   a:=dy/dx;
	 b:=y1-a*x1;
   v:=a*xo+b;
   if yo>v then result:=0 else result:=1;
	 end
	 else
	 begin
	 a:=dx/dy;
	 b:=x1-a*y1;
   v:=a*yo+b;
   if xo>v then result:=0 else result:=1;
	 end;
end;

function TriangleExists(p1,p2,p3:integer):boolean;
var i:integer;
begin
result:=true;
for i:=TrianglesCount-1 downto 0 do if ((p1=triangles[i].p1) or (p1=triangles[i].p2) or (p1=triangles[i].p3)) and
                                       ((p2=triangles[i].p1) or (p2=triangles[i].p2) or (p2=triangles[i].p3)) and
                                       ((p3=triangles[i].p1) or (p3=triangles[i].p2) or (p3=triangles[i].p3)) then exit;
result:=false;   
end;

function  SolveCircle(x1,y1,x2,y2,x3,y3:single):TCircle;
var ma,mb,dx1,dy1,dx2,dy2,dm:single;
begin
dx1:=x2-x1;dy1:=y2-y1;
dx2:=x3-x2;dy2:=y3-y2;
if abs(dx1)<0.01 then begin x1:=x1-0.1;dx1:=x2-x1;end;
if abs(dx2)<0.01 then begin x3:=x3+0.1;dx2:=x3-x2;end;
if abs(dy1)<0.01 then begin y1:=y1-0.1;dy1:=y2-y1;end;
if abs(dy2)<0.01 then begin y3:=y3+0.1;dy2:=y3-y2;end;
ma:=dy1/dx1;
mb:=dy2/dx2;
dm:=mb-ma;
if abs(dm)<0.0000001 then begin y3:=y3+0.1;dy2:=y3-y2;mb:=dy2/dx2;dm:=mb-ma;end;
result.xo:=(ma*mb*(y1-y3)+mb*(x1+x2)-ma*(x2+x3))*0.5/dm;
result.yo:=-1/mb*(result.xo-(x2+x3)*0.5)+(y2+y3)*0.5;
dx1:=x1-result.xo;
dy1:=y1-result.yo;
result.R:=sqrt(dx1*dx1+dy1*dy1);
end;

function FindPoint(r1,r2:integer):integer;
var i,j:integer;
cr:TCircle;
b:boolean;
x2,y2,v:single;
begin
result:=-1;
for i:=0 to pointsCount-1 do
 if (i<>r1) and (i<>r2) and (not TriangleExists(r1,r2,i)) then
   begin
   cr:=SolveCircle(points[r1].x,points[r1].y,points[r2].x,points[r2].y,points[i].x,points[i].y);
   b:=true;
   for j:=0 to pointsCount-1 do if (j<>r1) and (j<>r2) and (j<>i) then
      begin
      x2:=points[j].x-cr.xo;
      y2:=points[j].y-cr.yo;
      v:=sqrt(x2*x2+y2*y2);
      if v<cr.R then begin b:=false;break;end;
      end;
   if b then
      begin
      result:=i;
      exit;
      end;
   end;
end;

procedure FindFirstRib;
var i,j,k,n:integer;
st_1,st_0:boolean;
begin
for i:=0 to pointsCount-2 do
	 begin
	 for j:=i+1 to pointsCount-1 do
		  begin
	   	st_1:=false;
	  	st_0:=false;
	  	for k:=0 to pointsCount-1 do
		  	 begin
		  	 if (k<>i) and (k<>j) then
			  	  begin
			  	  n:=Side(i,j,k);
			  	  if n=1 then st_1:=true;
			  	  if n=0 then st_0:=true;
			  	  end;
			   end;
	  	if not (st_1 and st_0) then
			   begin
         Ribs[0].p1:=i;
		     Ribs[0].p2:=j;
		     RibsCount:=1;
         TrianglesCount:=0;
         exit;
			   end;
		  end;
	 end;
end;

procedure Triangulation;
var i,p1,p2,n:integer;
begin
FindFirstRib;
i:=0;
while (i<RibsCount) do
   begin
   p1:=Ribs[i].p1;
   p2:=Ribs[i].p2;
   n:=FindPoint(p1,p2);
   if n>=0 then
       begin
		   Ribs[RibsCount].p1:=p1;Ribs[RibsCount].p2:=n;
		   Ribs[RibsCount+1].p1:=p2;Ribs[RibsCount+1].p2:=n;
		   RibsCount:=RibsCount+2;
       triangles[TrianglesCount].p1:=p1;
       triangles[TrianglesCount].p2:=p2;
       triangles[TrianglesCount].p3:=n;
       TrianglesCount:=TrianglesCount+1;
		   end;
	 i:=i+1;
   end;
end;

end.
